
use lib "C:/Program Files (x86)/Cypress/Programmer/3.17";
use PSoC_Programmer1;
use strict;
use warnings;
use FileHandle;
use Getopt::Long;
use integer;
use sigtrap qw(die untrapped normal-signals stack-trace any error-signals);

#########################################
#  
# Initialize SWD connection
#
#########################################

my $pp = new PSoC_Programmer1;
$pp->start("C:/Program Files (x86)/Cypress/Programmer/3.17/ppcli");
my @ports = $pp->getports();
$pp->OpenPort($ports[0],"C:/Program Files (x86)/Cypress/Programmer/3.17");
$pp->SetProtocol(8);
$pp->DAP_Acquire();
$pp->PSoC3_DebugPortConfig(4);


#########################################
#  
# Get User Input File
#
#########################################

if (@ARGV > 1) {
   error("too many parameters");
}

my $input_file = shift @ARGV;

printf "Input file is $input_file\n \n";

#######################################
#
# Convert Intel Hex file to an array
#
#######################################

my @data = ();
my $addr = 0x00038000;

$pp->PSoC3_EraseSector(0x03, 0x02);
$pp->PSoC3_EraseSector(0x03, 0x03);

load_intel_hex(\@data, $input_file);

my $size = scalar(@data) - 1;

my $j = 0;

for ($j = 0; $j <= $size; $j++) {
	if (!defined($data[$j])) {
		$data[$j] = 0;
	}
}

my $rows = $size/256;

my $i = 0;

for ($i = 0; $i <= $rows; $i++) {
	if ($i == $rows) {
		my $zeros = 256 - ($size - 256*$rows + 1);
		my @zeros = (0) x $zeros;
		push (@data,0xA5,@zeros);
	}
	my @zeros1 = (0) x 32;
	my @new_data = @data[$i*256..(($i+1)*256 - 1)];
	push (@new_data,@zeros1);
	
	$pp->PSoC3_ProgramRow(3,(0x80 + $i),@new_data);
	
	my $result = $pp->PSoC3_ReadRow(3,(0x80 + $i),0);

	print "$result";
}

print "\n \n";

$pp->DAP_ReleaseChip();
$pp->ClosePort();

$pp->quit();
1;

#######################################
#
# Load Intel hex data into rom array
#
#######################################

sub load_intel_hex {
    my $r_array = shift;
    my $hex_file = shift;
	
    my $max_address = 0;

    my $in = new FileHandle $hex_file;
    if ($in) {
        my $rec_num = 0;
        while (my $rec = <$in>) {
            $rec_num++;
            $rec =~ s/\s+$//;
            if ($rec eq "") {
                next;
            }
            if ($rec !~ /^:/) {
                error("record $rec_num does not begin with hex record separator");
            }
            $rec =~ s/^://;
            if ($rec !~ /^[0-9a-fA-F]+$/) {
                error("record $rec_num does not consist entirely of hex digits");
            }
            if ((length $rec) % 2 != 0) {
                error("record $rec_num does not have an even number of hex digits");
            }
            my @rec_bytes = ();
            for (my $j = 0; $j < length $rec; $j += 2) {
                my $val = hex( substr( $rec, $j, 2));
                push @rec_bytes, $val;
            }
            my $check_sum = 0;
            foreach my $val (@rec_bytes) {
                $check_sum += $val;
            }
            if (($check_sum & 0xff) != 0) {
                error("record $rec_num has bad checksum");
            }
            if (@rec_bytes < 5) {
                error("record $rec_num is incomplete (under 5 data bytes)");
            }
            my $rec_len = $rec_bytes[0];
            if ($rec_len + 5 != @rec_bytes) {
                error("record $rec_num length field is invalid");
            }
            my $rec_address = ($rec_bytes[1] << 8) + $rec_bytes[2];

            my $rec_type = $rec_bytes[3];
            if ($rec_type == 0x01) {
                # End record
                last;
            }
            if ($rec_type == 0x04) {
                error("record $rec_num extended address records not supported");
            }
            if ($rec_type != 0x00) {
                error("record $rec_num unknown record type");
            }

            # record is good, load it into the rom array
			

            for (my $j = 4; $j < @rec_bytes - 1; $j++) {
                $$r_array[$rec_address++] = $rec_bytes[$j];
                if ($max_address < $rec_address) {
                    $max_address = $rec_address;
                }
            }
        }
    }

    return $max_address;
}


#"DVKProg5/FFFFFFFFFFFF"